# Movies Recommender Catalogue

Project for PA165 - Java Enterprise Applications

Movie recommender is a web application that allows users to rate movies and get recommendations based on their ratings.

## About Project

- **Name**: Movies Recommender Catalogue
- **Technologies**: Java 11, Maven, Spring Boot, JPA, Lombok
- **Developers**:
    - Filip Karniš @xkarnis - _Project Leader_
    - Pavol Baran @xbaran4
    - Michal Drobňák  @xdrobnak
- **[Assignment](./ASSIGNMENT.md)**


### Building the application
The application is built using maven. If you don't have maven available replace `mvn` with `./mvnw` or `./mvnw.cmd` on Windows. This applies for other commands as well. To build the application, run the following command in the root directory of the project:
```shell
mvn clean install
```
### Testing the application
To test the application, run the following command in the root directory of the project, this runs test in all submodules:
```shell
mvn test
```
### Running the application
The application consists of multiple microservices runnable as separate applications. To run a microservice, run the following from root directory of the project:
```shell
cd microservice # where microservice is on of [account, genre, movie, rating]
mvn spring-boot:run
```


## Components
The application consists of the following maven submodules:

### account
Represents a microservice for managing user accounts. Provides a REST API that allows for CRUD operations on accounts. For usage instructions, see [USAGE.md](./account/USAGE.md).


### core
Represents shared code between other modules. Contains DTOs, exceptions, and other classes that are used by other modules. This module is not runnable as a standalone application.

### genre
Represents a microservice for managing movie genres. Provides a REST API that allows for CRUD operations on genres. For usage instructions, see [USAGE.md](./genre/USAGE.md).

### movie
Represents a microservice for managing movies. Provides a REST API that allows for CRUD operations on movies. For usage instructions, see [USAGE.md](./movie/USAGE.md).

### rating
Represents a microservice for managing movie ratings. Provides a REST API that allows for CRUD operations on ratings. For usage instructions, see [USAGE.md](./rating/USAGE.md).

## Roles
System has two user roles - **Admin** and **Generic User**.

* **Admin** is a role for a system administrator. This user has the ability to create, read, update and delete movies or genres into the catalogue.
* **Generic User** is self-explanatory. This user can view read movie catalogue, create ratings for movies and ask the system to recommend similar movies to a certain (selected) movie.

## Entities
* **Account** - represents an account of an admin or generic user.
* **Movie** - represents a movie.
* **Rating** - represents ratings that correspond to certain movie.
* **Genre** - represents a genre of a movie.


## Class diagram for DTOs

![Class diagram](./uml/class-diagram.png)

## Use case diagram

![Use case diagram](./uml/use-case-diagram.png)