package cz.muni.pa165.movie.facade;

import cz.muni.pa165.core.api.movie.MovieBasicViewDto;
import cz.muni.pa165.core.api.movie.MovieDetailedViewDto;
import cz.muni.pa165.movie.mapper.MovieMapper;
import cz.muni.pa165.movie.service.MovieService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class MovieFacade {

    private final MovieService movieService;
    private final MovieMapper movieMapper;

    @Autowired
    public MovieFacade(MovieService movieService, MovieMapper movieMapper) {
        this.movieService = movieService;
        this.movieMapper = movieMapper;
    }

    @Transactional(readOnly = true)
    public MovieDetailedViewDto findById(Long id) {
        return movieMapper.mapToDetailViewDto(movieService.findById(id));
    }

    @Transactional(readOnly = true)
    public Page<MovieBasicViewDto> findAll(Pageable pageable) {
        return movieMapper.mapToPageDto(movieService.findAll(pageable));
    }

    @Transactional
    public MovieDetailedViewDto create(MovieDetailedViewDto movie) {
        return movieMapper.mapToDetailViewDto(
            movieService.create(movieMapper.mapToEntity(movie))
        );
    }

    @Transactional
    public MovieDetailedViewDto update(Long id, MovieDetailedViewDto movie) {
        movie.setId(id);
        return movieMapper.mapToDetailViewDto(
            movieService.update(movieMapper.mapToEntity(movie))
        );
    }

    @Transactional
    public void delete(Long id) {
        movieService.delete(movieService.findById(id));
    }
}
